@@ -2,16 +2,17 @@
 
 from __future__ import annotations
 
-import enum
 import random
 import re
 import string
+import subprocess
 import sys
 import warnings
+from dataclasses import replace
 from functools import lru_cache
 from pathlib import Path
 from re import Match, Pattern
-from typing import TYPE_CHECKING, Any, Callable, ClassVar
+from typing import TYPE_CHECKING, Any, Callable, ClassVar, Literal
 
 from griffe import (
     Alias,
@@ -27,11 +28,11 @@ from griffe import (
 )
 from jinja2 import TemplateNotFound, pass_context, pass_environment
 from markupsafe import Markup
-from mkdocs_autorefs.references import AutorefsHookInterface
+from mkdocs_autorefs import AutorefsHookInterface
 from mkdocstrings.loggers import get_logger
 
 if TYPE_CHECKING:
-    from collections.abc import Sequence
+    from collections.abc import Iterator, Sequence
 
     from griffe import Attribute, Class, Function, Module
     from jinja2 import Environment, Template
@@ -41,15 +42,6 @@ if TYPE_CHECKING:
 logger = get_logger(__name__)
 
 
-class Order(enum.Enum):
-    """Enumeration for the possible members ordering."""
-
-    alphabetical = "alphabetical"
-    """Alphabetical order."""
-    source = "source"
-    """Source code order."""
-
-
 def _sort_key_alphabetical(item: CollectorItem) -> Any:
     # chr(sys.maxunicode) is a string that contains the final unicode
     # character, so if 'name' isn't found on the object, the item will go to
@@ -64,18 +56,19 @@ def _sort_key_source(item: CollectorItem) -> Any:
     return item.lineno if item.lineno is not None else -1
 
 
+Order = Literal["alphabetical", "source"]
 order_map = {
-    Order.alphabetical: _sort_key_alphabetical,
-    Order.source: _sort_key_source,
+    "alphabetical": _sort_key_alphabetical,
+    "source": _sort_key_source,
 }
 
 
 def do_format_code(code: str, line_length: int) -> str:
-    """Format code using Black.
+    """Format code.
 
     Parameters:
         code: The code to format.
-        line_length: The line length to give to Black.
+        line_length: The line length.
 
     Returns:
         The same code, formatted.
@@ -83,7 +76,7 @@ def do_format_code(code: str, line_length: int) -> str:
     code = code.strip()
     if len(code) < line_length:
         return code
-    formatter = _get_black_formatter()
+    formatter = _get_formatter()
     return formatter(code, line_length)
 
 
@@ -118,7 +111,7 @@ def _format_signature(name: Markup, signature: str, line_length: int) -> str:
     # Black cannot format names with dots, so we replace
     # the whole name with a string of equal length
     name_length = len(name)
-    formatter = _get_black_formatter()
+    formatter = _get_formatter()
     formatable = f"def {'x' * name_length}{signature}: pass"
     formatted = formatter(formatable, line_length)
 
@@ -137,13 +130,13 @@ def do_format_signature(
     annotations: bool | None = None,
     crossrefs: bool = False,  # noqa: ARG001
 ) -> str:
-    """Format a signature using Black.
+    """Format a signature.
 
     Parameters:
         context: Jinja context, passed automatically.
         callable_path: The path of the callable we render the signature of.
         function: The function we render the signature of.
-        line_length: The line length to give to Black.
+        line_length: The line length.
         annotations: Whether to show type annotations.
         crossrefs: Whether to cross-reference types in the signature.
 
@@ -158,8 +151,7 @@ def do_format_signature(
         new_context = context.parent
     else:
         new_context = dict(context.parent)
-        new_context["config"] = dict(new_context["config"])
-        new_context["config"]["show_signature_annotations"] = annotations
+        new_context["config"] = replace(new_context["config"], show_signature_annotations=annotations)
 
     signature = template.render(new_context, function=function, signature=True)
     signature = _format_signature(callable_path, signature, line_length)
@@ -169,6 +161,7 @@ def do_format_signature(
             language="python",
             inline=False,
             classes=["doc-signature"],
+            linenums=False,
         ),
     )
 
@@ -198,13 +191,13 @@ def do_format_attribute(
     *,
     crossrefs: bool = False,  # noqa: ARG001
 ) -> str:
-    """Format an attribute using Black.
+    """Format an attribute.
 
     Parameters:
         context: Jinja context, passed automatically.
         attribute_path: The path of the callable we render the signature of.
         attribute: The attribute we render the signature of.
-        line_length: The line length to give to Black.
+        line_length: The line length.
         crossrefs: Whether to cross-reference types in the signature.
 
     Returns:
@@ -213,7 +206,7 @@ def do_format_attribute(
     env = context.environment
     # TODO: Stop using `do_get_template` when `*.html` templates are removed.
     template = env.get_template(do_get_template(env, "expression"))
-    annotations = context.parent["config"]["show_signature_annotations"]
+    annotations = context.parent["config"].show_signature_annotations
 
     signature = str(attribute_path).strip()
     if annotations and attribute.annotation:
@@ -230,6 +223,7 @@ def do_format_attribute(
             language="python",
             inline=False,
             classes=["doc-signature"],
+            linenums=False,
         ),
     )
 
@@ -332,26 +326,47 @@ def do_multi_crossref(text: str, *, code: bool = True) -> Markup:
     return Markup(text).format(**variables)
 
 
-def do_split_path(path: str, full_path: str) -> list[tuple[str, str]]:
+_split_path_re = re.compile(r"([.(]?)([\w]+)(\))?")
+_splitable_re = re.compile(r"[().]")
+
+
+def do_split_path(path: str, full_path: str) -> Iterator[tuple[str, str, str, str]]:
     """Split object paths for building cross-references.
 
     Parameters:
         path: The path to split.
+        full_path: The full path, used to compute correct paths for each part of the path.
 
-    Returns:
-        A list of pairs (title, full path).
+    Yields:
+        4-tuples: prefix, word, full path, suffix.
     """
-    if "." not in path:
-        return [(path, full_path)]
-    pairs = []
-    full_path = ""
-    for part in path.split("."):
-        if full_path:
-            full_path += f".{part}"
-        else:
-            full_path = part
-        pairs.append((part, full_path))
-    return pairs
+    # Path is a single word, yield full path directly.
+    if not _splitable_re.search(path):
+        yield ("", path, full_path, "")
+        return
+
+    current_path = ""
+    if path == full_path:
+        # Split full path and yield directly without storing data in a dict.
+        for match in _split_path_re.finditer(full_path):
+            prefix, word, suffix = match.groups()
+            current_path = f"{current_path}{prefix}{word}{suffix or ''}" if current_path else word
+            yield prefix or "", word, current_path, suffix or ""
+        return
+
+    # Split full path first to store tuples in a dict.
+    elements = {}
+    for match in _split_path_re.finditer(full_path):
+        prefix, word, suffix = match.groups()
+        current_path = f"{current_path}{prefix}{word}{suffix or ''}" if current_path else word
+        elements[word] = (prefix or "", word, current_path, suffix or "")
+
+    # Then split path and pick tuples from the dict.
+    first = True
+    for match in _split_path_re.finditer(path):
+        prefix, word, current_path, suffix = elements[match.group(2)]
+        yield "" if first else prefix, word, current_path, suffix
+        first = False
 
 
 def _keep_object(name: str, filters: Sequence[tuple[Pattern, bool]]) -> bool:
@@ -432,12 +447,59 @@ def do_filter_objects(
 
 
 @lru_cache(maxsize=1)
-def _get_black_formatter() -> Callable[[str, int], str]:
+def _get_formatter() -> Callable[[str, int], str]:
+    for formatter_function in [
+        _get_black_formatter,
+        _get_ruff_formatter,
+    ]:
+        if (formatter := formatter_function()) is not None:
+            return formatter
+
+    logger.info("Formatting signatures requires either Black or Ruff to be installed.")
+    return lambda text, _: text
+
+
+def _get_ruff_formatter() -> Callable[[str, int], str] | None:
+    try:
+        from ruff.__main__ import find_ruff_bin
+    except ImportError:
+        return None
+
+    try:
+        ruff_bin = find_ruff_bin()
+    except FileNotFoundError:
+        ruff_bin = "ruff"
+
+    def formatter(code: str, line_length: int) -> str:
+        try:
+            completed_process = subprocess.run(  # noqa: S603
+                [
+                    ruff_bin,
+                    "format",
+                    "--config",
+                    f"line-length={line_length}",
+                    "--stdin-filename",
+                    "file.py",
+                    "-",
+                ],
+                check=True,
+                capture_output=True,
+                text=True,
+                input=code,
+            )
+        except subprocess.CalledProcessError:
+            return code
+        else:
+            return completed_process.stdout
+
+    return formatter
+
+
+def _get_black_formatter() -> Callable[[str, int], str] | None:
     try:
         from black import InvalidInput, Mode, format_str
     except ModuleNotFoundError:
-        logger.info("Formatting signatures requires Black to be installed.")
-        return lambda text, _: text
+        return None
 
     def formatter(code: str, line_length: int) -> str:
         mode = Mode(line_length=line_length)
@@ -498,11 +560,20 @@ def do_as_attributes_section(
     Returns:
         An attributes docstring section.
     """
+
+    def _parse_docstring_summary(attribute: Attribute) -> str:
+        if attribute.docstring is None:
+            return ""
+        line = attribute.docstring.value.split("\n", 1)[0]
+        if ":" in line and attribute.docstring.parser_options.get("returns_type_in_property_summary", False):
+            _, line = line.split(":", 1)
+        return line
+
     return DocstringSectionAttributes(
         [
             DocstringAttribute(
                 name=attribute.name,
-                description=attribute.docstring.value.split("\n", 1)[0] if attribute.docstring else "",
+                description=_parse_docstring_summary(attribute),
                 annotation=attribute.annotation,
                 value=attribute.value,  # type: ignore[arg-type]
             )
@@ -528,7 +599,7 @@ def do_as_functions_section(
     Returns:
         A functions docstring section.
     """
-    keep_init_method = not context.parent["config"]["merge_init_into_class"]
+    keep_init_method = not context.parent["config"].merge_init_into_class
     return DocstringSectionFunctions(
         [
             DocstringFunction(
@@ -592,7 +663,7 @@ def do_as_modules_section(
                 description=module.docstring.value.split("\n", 1)[0] if module.docstring else "",
             )
             for module in modules
-            if not check_public or module
+            if not check_public or module.is_public
         ],
     )
 
